{
/**********************************************************/
/*                      IpList.pas  V1                    */
/*                                                        */
/* Last release 21/05/2003                Sylvain Taufour */
/*                                sylvain.taufour@free.fr */
/**********************************************************/
}



unit IpList;

interface

uses
  Classes, WinSock, SysUtils, Ras, IdHTTP, StrUtils, Registry, IdHTTPHeaderInfo;

function GetProxySettings(var ProxyServeur: String; var ProxyPort: Integer): Boolean;
procedure GetIPList(ToList : TStrings);
function GetInternetIp(ProxyConf: TIdProxyConnectionInfo = nil): String;

type
  TGetMyIpTh = class(TThread)
  private
    FIp: String;
    FProxyConf: TIdProxyConnectionInfo;
    function GetProxyConf: TIdProxyConnectionInfo;
    procedure SetProxyConf(const Value: TIdProxyConnectionInfo);
  protected
    procedure Execute; override;
  public
    property Ip: String read FIp;
    property ProxyConf: TIdProxyConnectionInfo read GetProxyConf write SetProxyConf;
  end;

  TGetMyIp = class
  private
    FProxyConf: TIdProxyConnectionInfo;
    FIp: String;
    IpThread: TGetMyIpTh;
    FOnHaveIp: TNotifyEvent;
    InProgress: Boolean;
    procedure EndThread(Sender: TObject);
    function GetProxyConf: TIdProxyConnectionInfo;
    procedure SetProxyConf(const Value: TIdProxyConnectionInfo);
  protected
  public
    constructor Create; overload;
    destructor Destroy; overload;
    procedure GetIp;
    property Ip: String read FIp;
    property OnHaveIp: TNotifyEvent read FOnHaveIp write FOnHaveIp;
    property ProxyConf: TIdProxyConnectionInfo read GetProxyConf write SetProxyConf;
  end;

implementation

function GetInternetIp(ProxyConf: TIdProxyConnectionInfo = nil): String;
var
  S, ProxS: String;
  ProxP: Integer;
begin
  GetInternetIp := '';

  if (ProxyConf <> nil) or GetProxySettings(ProxS, ProxP) then
  begin
    try
      with TIdHTTP.Create(nil) do
      begin
        try
          with ProxyParams do
          begin
            if ProxyConf = nil then
            begin
              ProxyServer := ProxS;
              ProxyPort := ProxP;
              BasicAuthentication := True;
            end
            else
            begin
              with ProxyParams do
              begin
                ProxyPassword := ProxyConf.ProxyPassword;
                ProxyPort := ProxyConf.ProxyPort;
                ProxyServer := ProxyConf.ProxyServer;
                ProxyUsername := ProxyConf.ProxyUsername;
                if (ProxyPassword = '') and (ProxyUsername = '') then
                  BasicAuthentication := True;
              end;
            end;
          end;

          with TStringList.Create do
          try
            Text := Get('http://checkip.dyndns.org');
            S := Trim(RightStr(Strings[3], Length(Strings[3]) - Pos(':', Strings[3])));
          finally
            Free;
          end;
        finally
          Free;
        end;
      end;
    except
      S := '';
    end;
  end
  else
    S := RasGetIPAddress;
    
  result := S;
end;

procedure GetIPList(ToList : TStrings);
type 
  TaPInAddr = array [0..255] of PInAddr;
  PaPInAddr = ^TaPInAddr;
var
  pptr: PaPInAddr;
  I: Integer;
  HostEnt: PHostEnt;
  Name: String;
  WSAData: TWSAData;
begin
  WSAStartup(2, WSAData);
  SetLength(Name, 255);
  Gethostname(PChar(Name), 255);
  SetLength(Name, StrLen(PChar(Name)));
  HostEnt := gethostbyname(PChar(Name));
  if HostEnt <> nil then
  begin
    pptr := PaPInAddr(HostEnt^.h_addr_list);
    I := 0;
    while pptr^[I] <> nil do begin
      ToList.Add(StrPas(inet_ntoa(pptr^[I]^)));
      Inc(I);
    end;
  end; 
  WSACleanup;
end; 

function GetProxySettings(var ProxyServeur: String; var ProxyPort: Integer): Boolean;
var
  Proxy: String;
begin
  ProxyServeur := '';
  ProxyPort := 80;
  Result := False;

  with TRegistry.Create do
  try
    if OpenKeyReadOnly('Software\Microsoft\Windows\CurrentVersion\Internet Settings') then
    begin
      try
        Result := (ReadInteger('ProxyEnable') = 1);
        if Result then
        begin
          Proxy := ReadString('ProxyServer');
          ProxyServeur := Copy(Proxy, 1, Pos(':', Proxy) - 1);
          ProxyPort := StrToIntDef(Copy(Proxy, Pos(':', Proxy) + 1, 5), 80);
        end;
      except
        raise;
      end;
    end;
  finally
    Free;
  end;
end;

{ TGetMyIpTh }

procedure TGetMyIpTh.Execute;
begin
  FIp := GetInternetIp(FProxyConf)
end;

function TGetMyIpTh.GetProxyConf: TIdProxyConnectionInfo;
begin
  Result := FProxyConf;
end;

procedure TGetMyIpTh.SetProxyConf(const Value: TIdProxyConnectionInfo);
begin
  FProxyConf := Value;
end;

{ TGetMyIp }

constructor TGetMyIp.Create;
begin
  inherited Create;

  //FProxyConf := TIdProxyConnectionInfo.Create;
  InProgress := False;
end;

destructor TGetMyIp.Destroy;
begin
//  FProxyConf.Free;
  
  inherited Destroy;
end;

procedure TGetMyIp.EndThread(Sender: TObject);
begin
  InProgress := False;
  FIp := IpThread.Ip;
  IpThread := nil;
  if Assigned(FOnHaveIp) then
    FOnHaveIp(self);
end;

procedure TGetMyIp.GetIp;
begin
  if not InProgress then
  begin
    IpThread :=  TGetMyIpTh.Create(True);
    with IpThread do
    begin
      OnTerminate := EndThread;
      FreeOnTerminate := True;
      ProxyConf := Self.FProxyConf;
      InProgress := True;
      Resume;
    end;
  end;
end;

function TGetMyIp.GetProxyConf: TIdProxyConnectionInfo;
begin
  Result := FProxyConf;
end;

procedure TGetMyIp.SetProxyConf(const Value: TIdProxyConnectionInfo);
begin
  FProxyConf := Value;
end;

end.
