{
/**********************************************************/
/*                       MAC.pas  V1                      */
/*                                                        */
/* Last release 21/05/2003                Sylvain Taufour */
/*                                sylvain.taufour@free.fr */
/**********************************************************/
}

unit MAC;

interface

uses
  IpHlpApi, Windows, Sysutils;

procedure Get_IfTableMIB(var MIBIfArray: TMIBIfArray);
function MacAddr2Str(MacAddr: TMacAddress; size: integer): string;
function GetMac(No: Integer): String;
function NbMac: Integer;

implementation

function GetMac(No: Integer): String;
var
  MibArr: IpHlpAPI.TMibIfArray;
begin
  Get_IfTableMIB(MibArr);
  result := MacAddr2Str(TMacAddress(MIBArr[No].bPhysAddr), MIBArr[No].dwPhysAddrLen);
end;

procedure Get_IfTableMIB( var MIBIfArray: TMIBIfArray );
var
  i, Error, TableSize: integer;
  pBuf: PChar;
  NumEntries: DWORD;
begin
  TableSize := 0;
  pBuf := '';
  NumEntries := 0;
   // first call: get memsize needed
  Error := GetIfTable(PTMibIfTable(pBuf), @TableSize, false );
  if Error <> ERROR_INSUFFICIENT_BUFFER then
    EXIT;
  GetMem(pBuf, TableSize);

   // get table pointer
  Error := GetIfTable(PTMibIfTable(pBuf), @TableSize, false);
  if Error = NO_ERROR then
  begin
    NumEntries := PTMibIfTable(pBuf)^.dwNumEntries;
    if NumEntries > 0 then
    begin
      SetLength(MIBIfArray, NumEntries);
      inc(pBuf, SizeOf(NumEntries));
      for i := 0 to pred(NumEntries) do
      begin
        MIBIfArray[i] := PTMibIfRow(pBuf)^;
        inc(pBuf, SizeOf(TMIBIfRow));
      end;
    end
  end;
  dec(pBuf, SizeOf(DWORD) + NumEntries * SizeOf(TMIBIfRow));
  FreeMem(pBuf);
end;

function MacAddr2Str(MacAddr: TMacAddress; size: integer): string;
var
  i: integer;
begin
  if Size = 0 then
  begin
    Result := '00-00-00-00-00-00';
    EXIT;
  end
  else Result := '';
  //
  for i := 1 to Size do
    Result := Result + IntToHex(MacAddr[i], 2) + '-';
  Delete(Result, Length(Result), 1);
end;

function NbMac: Integer;
var
  MibArr: IpHlpAPI.TMibIfArray;
begin
  Get_IfTableMIB(MibArr);
  result := High(MIBArr);
end;

end.
